--
-- xmonad example config file.
--
-- A template showing all available configuration hooks,
-- and how to override the defaults in your own xmonad.hs conf file.
--
-- Normally, you'd only override those defaults you care about.
--

import XMonad
import System.Exit
import System.IO

import XMonad.Util.Run
import XMonad.Util.Scratchpad
import XMonad.Util.Loggers
import XMonad.Actions.WindowBringer
import XMonad.Layout.WindowNavigation
import XMonad.Config.Azerty
import XMonad.Hooks.DynamicLog
import XMonad.Hooks.ManageDocks
--import XMonad.Util.EZConfig(additionalKeys)
import Graphics.X11.ExtraTypes.XF86
import XMonad.Hooks.EwmhDesktops
import XMonad.Hooks.SetWMName
import XMonad.Hooks.ManageHelpers

import XMonad.Layout.MosaicAlt
import XMonad.Layout.ResizableTile
import XMonad.Layout.Tabbed
import XMonad.Layout.NoBorders
import XMonad.Layout.Grid
--import XMonad.Layout.TwoPane
import XMonad.Layout.Fullscreen

import XMonad.Layout.Maximize

-- fullscreen attempt:
--import XMonad.Layout.Spacing
--import XMonad.Layout.Gaps
--import XMonad.Layout.ResizableTile
-----import XMonad.Layout.NoBorders
--import XMonad.Layout.Fullscreen (fullscreenFull)
--import XMonad.Layout.Cross(simpleCross)
--import XMonad.Layout.Spiral(spiral)
--import XMonad.Layout.ThreeColumns
--import XMonad.Layout.MultiToggle
--import XMonad.Layout.MultiToggle.Instances
--import XMonad.Layout.IndependentScreens

-- import System.Cmd
import System.Process

import qualified XMonad.StackSet as W
import qualified Data.Map        as M

-- The preferred terminal program, which is used in a binding below and by
-- certain contrib modules.
--
myTerminal = "uxterm -bg \"grey10\" -fg grey +sb"
myUrxvt = "urxvt -bg \"grey10\" -fg grey +sb"
myBrowser = "firefox"
myChat    = "pidgin"
myExecuteCommand    = "gmrun"
myPassCommand    = "passmenu"
myFileManager  = "ranger"
myMail    = "uxterm -e mutt" -- -F ~/.muttrc.enstb"
myLock    = "xautolock -time 3 -locker 'i3lock -d -i ~/images/gentoo/gentoo-larry-bg-1280x800.png -t' & xautolock -locknow -locker 'i3lock -d -i ~/images/gentoo/gentoo-larry-bg-1280x800.png -t'"
myImgurScreenshot = "imgur-screenshot.sh" -- && mplayer /usr/share/sounds/freedesktop/stereo/screen-capture.oga"
myPointer =  "find-cursor --size 150 --distance 40 --wait 400 --line-width 4 --color red"

-- Width of the window border in pixels.
--
myBorderWidth   = 1

-- modMask lets you specify which modkey you want to use. The default
-- is mod1Mask ("left alt").  You may also consider using mod3Mask
-- ("right alt"), which does not conflict with emacs keybindings. The
-- "windows key" is usually mod4Mask.
--
myModMask       = mod1Mask
myAltModMask       = mod4Mask

-- The mask for the numlock key. Numlock status is "masked" from the
-- current modifier status, so the keybindings will work with numlock on or
-- off. You may need to change this on some systems.
--
-- You can find the numlock modifier by running "xmodmap" and looking for a
-- modifier with Num_Lock bound to it:
--
-- > $ xmodmap | grep Num
-- > mod2        Num_Lock (0x4d)
--
-- Set numlockMask = 0 if you don't have a numlock key, or want to treat
-- numlock status separately.
--
myNumlockMask   = mod2Mask

-- The default number of workspaces (virtual screens) and their names.
-- By default we use numeric strings, but any string may be used as a
-- workspace name. The number of workspaces is determined by the length
-- of this list.
--
-- A tagging example:
--
-- > workspaces = ["web", "irc", "code" ] ++ map show [4..9]
--
--myWorkspaces    = ["1","2","3","4","5","utils","web","mail","chat"]
myWorkspaces    = ["1","2","3","4","5","6","7","8","9"]
--myWorkspaces    = ["\61612","\61899","\61947","\61635","\61502","\61501","\61705","\61564","\62150","\61872"]

-- Border colors for unfocused and focused windows, respectively.
--
myNormalBorderColor  = "#006633" --"#0033cc" -- "#dddddd"
myFocusedBorderColor = "#ff0000"

------------------------------------------------------------------------
-- Key bindings. Add, modify or remove key bindings here.
--
myKeys conf@(XConfig {XMonad.modMask = modMask}) = M.fromList $

    -- launch a terminal
    [ ((modMask .|. shiftMask, xK_Return), spawn $ XMonad.terminal conf)
    , ((myAltModMask .|. shiftMask, xK_Return), spawn $ XMonad.terminal conf)

    -- my custom bindings
    , ((myAltModMask, xK_u), spawn $ XMonad.terminal conf)
    --, ((myAltModMask, xK_a), spawn $ myTerminology)
    , ((myAltModMask, xK_a), spawn $ myUrxvt)
    , ((myAltModMask, xK_f), spawn $ myBrowser)
    -- , ((myAltModMask, xK_k), spawn $ myBrowser2)
    -- , ((myAltModMask, xK_c), spawn $ myBrowser3)
    , ((myAltModMask, xK_g), spawn $ myChat)
    , ((myAltModMask, xK_t), spawn $ myMail)
    -- , ((myAltModMask .|. shiftMask, xK_s), spawn $ mySSH)
    , ((myAltModMask, xK_x), spawn $ myExecuteCommand)
    , ((myAltModMask, xK_z), spawn $ myPassCommand )
    , ((myAltModMask, xK_m), spawn $ myFileManager)
    , ((myAltModMask, xK_l), spawn $ myLock)
    -- attempt to have something with fullscreen:
--    , ((myAltModMask, xK_b), sendMessage ToggleGaps >> spawn "polybar-msg cmd toggle")
    , ((myAltModMask, xK_c), spawn $ myImgurScreenshot)
    , ((myAltModMask, xK_p), spawn $ myPointer)
    -- touche "courriel" sur un bépo TypeMatrix 2030
    --, ((0, 0x1008ff19), spawn $ myPointer)
    -- touche PrintScr keysym 0xff61, Print 
    , ((0, 0xff61), spawn $ myPointer)
    
    -- launch dmenu
    , ((modMask,               xK_p     ), spawn "exe=`dmenu_path | dmenu` && eval \"exec $exe\"")

    -- scratch pad
    , ((modMask,      xK_backslash), scratchpadSpawnAction conf)
    , ((modMask,      xK_0), scratchpadSpawnAction conf)
    , ((myAltModMask, xK_backslash), scratchpadSpawnAction conf)
    , ((myAltModMask, xK_0), scratchpadSpawnAction conf)
    --"zooming" (import Maximize
    , ((modMask, xK_b), withFocused ( sendMessage . maximizeRestore))
    , ((myAltModMask, xK_b), withFocused ( sendMessage . maximizeRestore))
    --test
    , ((myAltModMask,         0x69), scratchpadSpawnAction conf)
    --, ((myAltModMask,         0xffeb), scratchpadSpawnAction conf)
    , ((myAltModMask,         0xe0), scratchpadSpawnAction conf)
    , ((modMask,         0xe0), scratchpadSpawnAction conf)

    -- window navigation
    --, ((modMask,                 xK_Right), sendMessage $ Go R)
    --, ((modMask,                 xK_Left ), sendMessage $ Go L)
    --, ((modMask,                 xK_Up   ), sendMessage $ Go U)
    --, ((modMask,                 xK_Down ), sendMessage $ Go D)
    --, ((modMask .|. controlMask, xK_Right), sendMessage $ Swap R)
    --, ((modMask .|. controlMask, xK_Left ), sendMessage $ Swap L)
    --, ((modMask .|. controlMask, xK_Up   ), sendMessage $ Swap U)
    --, ((modMask .|. controlMask, xK_Down ), sendMessage $ Swap D)

    -- launch xlock
    , ((modMask .|. controlMask, xK_l     ), spawn myLock)
    , ((myAltModMask .|. controlMask, xK_l     ), spawn myLock)

    -- launch gmrun
    , ((modMask .|. shiftMask, xK_x     ), spawn "gmrun")

     -- imgur-screenshot
    , ((modMask .|. shiftMask, xK_c     ), spawn myImgurScreenshot)

    -- close focused window 
    , ((modMask .|. shiftMask, xK_c     ), kill)
    --, ((myAltModMask .|. shiftMask, xK_c     ), kill)

     -- Rotate through the available layout algorithms
    , ((modMask,               xK_space ), sendMessage NextLayout)
    --, ((myAltModMask,               xK_space ), sendMessage NextLayout)

    --  Reset the layouts on the current workspace to default
    , ((modMask .|. shiftMask, xK_space ), setLayout $ XMonad.layoutHook conf)
    --, ((myAltModMask .|. shiftMask, xK_space ), setLayout $ XMonad.layoutHook conf)

    -- Resize viewed windows to the correct size
    , ((modMask,               xK_n     ), refresh)
    --, ((myAltModMask,               xK_n     ), refresh)

    -- Move focus to the next window
    , ((modMask,               xK_Tab   ), windows W.focusDown)
    --, ((myAltModMask,               xK_Tab   ), windows W.focusDown)

    -- Move focus to the next window
    , ((modMask,               xK_j     ), windows W.focusDown)
    --, ((myAltModMask,               xK_j     ), windows W.focusDown)

    -- Move focus to the previous window
    , ((modMask,               xK_k     ), windows W.focusUp  )
    --, ((myAltModMask,               xK_k     ), windows W.focusUp  )

    -- Move focus to the master window
    , ((modMask,               xK_m     ), windows W.focusMaster  )
    --, ((myAltModMask,               xK_m     ), windows W.focusMaster  )

    -- Swap the focused window and the master window
    , ((modMask,               xK_Return), windows W.swapMaster)
    -- ((myAltModMask,               xK_Return), windows W.swapMaster)

    -- Swap the focused window with the next window
    , ((modMask .|. shiftMask, xK_j     ), windows W.swapDown  )
    --, ((myAltModMask .|. shiftMask, xK_j     ), windows W.swapDown  )

    -- Swap the focused window with the previous window
    , ((modMask .|. shiftMask, xK_k     ), windows W.swapUp    )
    --, ((myAltModMask .|. shiftMask, xK_k     ), windows W.swapUp    )

    -- Shrink the master area
    , ((modMask,               xK_h     ), sendMessage Shrink)
    --, ((myAltModMask,               xK_h     ), sendMessage Shrink)

    -- Expand the master area
    , ((modMask,               xK_l     ), sendMessage Expand)
    --, ((myAltModMask,               xK_l     ), sendMessage Expand)

    -- Push window back into tiling
    , ((modMask,               xK_t     ), withFocused $ windows . W.sink)
    --, ((myAltModMask,               xK_t     ), withFocused $ windows . W.sink)

    -- Increment the number of windows in the master area
    , ((modMask              , xK_comma ), sendMessage (IncMasterN 1))
    --, ((myAltModMask, xK_comma ), sendMessage (IncMasterN 1))

    -- Deincrement the number of windows in the master area
    , ((modMask              , xK_period), sendMessage (IncMasterN (-1)))
    --, ((myAltModMask, xK_period), sendMessage (IncMasterN (-1)))

    -- toggle the status bar gap
    -- TODO, update this binding with avoidStruts , ((modMask              , xK_b     ),

    -- Quit xmonad
    , ((modMask .|. shiftMask, xK_q     ), io (exitWith ExitSuccess))

    -- Restart xmonad
    , ((modMask              , xK_q     ), restart "xmonad" True)
    ]
    ++

    -- function keys
    [(( 0, xF86XK_AudioLowerVolume ), spawn "amixer set Master 2-")
    , ((0, xF86XK_AudioRaiseVolume ), spawn "amixer set Master 2+")
    , ((0, xF86XK_AudioMute        ), spawn "amixer set Master toggle")
    --, ((0, xF86XK_AudioMicMute     ), spawn "amixer set ")
    , ((0, xF86XK_MonBrightnessUp  ), spawn "xbacklight -inc 5")
    , ((0, xF86XK_MonBrightnessDown), spawn "xbacklight -dec 5")
    ]
    ++

    --
    -- mod-[1..9], Switch to workspace N
    -- mod-shift-[1..9], Move client to workspace N
    --
    [((m .|. modMask, k), windows $ f i)
        | (i, k) <- zip (XMonad.workspaces conf) [xK_1 .. xK_9]
        , (f, m) <- [(W.greedyView, 0), (W.shift, shiftMask)]]
    ++

    [((m .|. myAltModMask, k), windows $ f i)
        | (i, k) <- zip (XMonad.workspaces conf) [xK_1 .. xK_9]
        , (f, m) <- [(W.greedyView, 0), (W.shift, shiftMask)]]
    ++

    -- bépo conf
    [((m .|. myAltModMask, k), windows $ f i)
        | (i, k) <- zip (XMonad.workspaces conf) [0x26,0xe9,0x22,0x27,0x28,0x2d,0xe8,0x5f,0xe7,0xe0,xK_a,xK_z,xK_e,xK_r] --azerty
--        | (i, k) <- zip (XMonad.workspaces conf) [0x22,0xab,0xbb,0x28,0x29,0x40,0x2b,0x2d,0x2f,0x2a,xK_a,xK_z,xK_e,xK_r] --bepo
        , (f, m) <- [(W.greedyView, 0), (W.shift, shiftMask)]]
    ++

    --
    -- mod-{w,e,r}, Switch to physical/Xinerama screens 1, 2, or 3
    -- mod-shift-{w,e,r}, Move client to screen 1, 2, or 3
    --
    [((m .|. modMask, key), screenWorkspace sc >>= flip whenJust (windows . f))
        | (key, sc) <- zip [xK_w, xK_e, xK_r] [0..]
        , (f, m) <- [(W.view, 0), (W.shift, shiftMask)]]
    ++

    [((m .|. myAltModMask, key), screenWorkspace sc >>= flip whenJust (windows . f))
        | (key, sc) <- zip [xK_w, xK_e, xK_r] [0..]
        , (f, m) <- [(W.view, 0), (W.shift, shiftMask)]]


------------------------------------------------------------------------
-- Mouse bindings: default actions bound to mouse events
--
myMouseBindings (XConfig {XMonad.modMask = modMask}) = M.fromList $

    -- mod-button1, Set the window to floating mode and move by dragging
    [ ((modMask, button1), (\w -> focus w >> mouseMoveWindow w))

    -- mod-button2, Raise the window to the top of the stack
    , ((modMask, button2), (\w -> focus w >> windows W.swapMaster))

    -- mod-button3, Set the window to floating mode and resize by dragging
    , ((modMask, button3), (\w -> focus w >> mouseResizeWindow w))

    -- you may also bind events to the mouse scroll wheel (button4 and button5)
    ]

------------------------------------------------------------------------
-- Layouts:

-- You can specify and transform your layouts by modifying these values.
-- If you change layout bindings be sure to use 'mod-shift-space' after
-- restarting (with 'mod-q') to reset your layout state to the new
-- defaults, as xmonad preserves your old layout settings by default.
--
-- The available layouts.  Note that each layout is separated by |||,
-- which denotes layout choice.
--
myLayout = smartBorders $ maximize (tiled ||| Mirror tiled ||| simpleTabbed ||| Full) -- ||| Grid
-- fullscreen attempt
--myLayout = spacingRaw True (Border 0 5 5 5) True (Border 5 5 5 5) True $ mkToggle (NBFULL ?? NOBORDERS ?? EOT) $ tiled ||| Mirror tiled ||| spiral (6/7)  ||| ThreeColMid 1 (3/100) (1/2) ||| Full
--myLayout = avoidStruts (tiled ||| Mirror tiled ||| Full) ||| Full
  where
     -- default tiling algorithm partitions the screen into two panes
     tiled   = Tall nmaster delta ratio

     -- The default number of windows in the master pane
     nmaster = 1

     -- Default proportion of screen occupied by master pane
     ratio   = 1/2

     -- Percent of screen to increment by when resizing panes
     delta   = 3/100

------------------------------------------------------------------------
-- Window rules:

-- Execute arbitrary actions and WindowSet manipulations when managing
-- a new window. You can use this to, for example, always float a
-- particular program, or have a client always appear on a particular
-- workspace.
--
-- To find the property name associated with a program, use
-- > xprop | grep WM_CLASS
-- and click on the client you're interested in.
--
-- To match on the WM_NAME, you can use 'title' in the same way that
-- 'className' and 'resource' are used below.
--
myManageHook = composeAll
    [ className =? "MPlayer"        --> doFloat
    , className =? "Gimp"           --> doFloat
    , className =? "Gmrun"          --> doFloat
    , className =? "stalonetray"    --> doIgnore
    , resource  =? "desktop_window" --> doIgnore
    , resource  =? "kdesktop"       --> doIgnore ]

-- Whether focus follows the mouse pointer.
myFocusFollowsMouse :: Bool
myFocusFollowsMouse = True


------------------------------------------------------------------------
-- Status bars and logging

-- Perform an arbitrary action on each internal state change or X event.
-- See the 'DynamicLog' extension for examples.
--
-- To emulate dwm's status bar
--
-- > logHook = dynamicLogDzen

myLogHook dest = do
        dynamicLogWithPP $ xmobarPP 
            { ppOutput = hPutStrLn dest
            , ppTitle = xmobarColor "green" "" . shorten 100 --50
            , ppSort = fmap (. scratchpadFilterOutWorkspace) $ ppSort xmobarPP
            , ppVisible = \x -> ""
            , ppHidden = \x -> ""
            , ppLayout = \x -> ""
            }             

------------------------------------------------------------------------
-- Startup hook

-- Perform an arbitrary action each time xmonad starts or is restarted
-- with mod-q.  Used by, e.g., XMonad.Layout.PerWorkspace to initialize
-- per-workspace layout choices.
--
-- By default, do nothing.
-- myStartupHook = return ()
-- old way to lie to Java about what window manager is used. Warning: break
-- recent version of gtk3, use _JAVA_AWT_WM_NONREPARENTING=1 variable instead
-- myStartupHook        = setWMName "LG3D"
myStartupHook        = do
  spawn "xsetroot -cursor_name right_ptr"
  spawn "feh --bg-tile /home/jcbach/.wallpapers/wallpaper.jpg"
--  spawn "sh ~/.TouchScreen.sh stop"
  spawn "[[ -f ~/.Xresources ]] && xrdb -merge ~/.Xresources"
  spawn "xmodmap ~/.Xmodmap"
  spawn "$HOME/keyboards.sh"
  spawn "xhost +local:root"
  -- location = IMTA (Plouzane)
  spawn "ps cax | grep redshift &>1 ; if ! [ $? -eq 0 ]; then redshift -l 48.35865:-4.57096 -t 6500:3700; fi"
  spawn "ps cax | grep stalonetray ; if ! [ $? -eq 0 ]; then stalonetray; fi"
  spawn "xautolock -time 3 -locker 'i3lock -d -i ~/images/gentoo/gentoo-larry-bg-1280x800.png -t'"
--  spawn "ps cax | grep ssh-agent &>1 ; if ! [ $? -eq 0 ]; then ssh-agent;fi"
  spawn "nm-applet"

------------------------------------------------------------------------
-- Now run xmonad with all the defaults we set up.

-- Run xmonad with the settings you specify. No need to modify this.
--

main = do
    xmproc <- spawnPipe "xmobar $XDG_CONFIG_HOME/xmobar/xmobarrc"
    xmonad $ fullscreenSupport $ def {
      -- simple stuff
        terminal           = myTerminal,
        focusFollowsMouse  = myFocusFollowsMouse,
        borderWidth        = myBorderWidth,
        modMask            = myModMask,
        --numlockMask        = myNumlockMask,
        workspaces         = myWorkspaces,
        normalBorderColor  = myNormalBorderColor,
        focusedBorderColor = myFocusedBorderColor,

      -- key bindings
        keys               = myKeys,
        mouseBindings      = myMouseBindings,

      -- hooks, layouts
        manageHook         = myManageHook,
        layoutHook         = avoidStruts myLayout,
        --layoutHook         = smartBorders . avoidStruts $ myLayout,
        handleEventHook    = mconcat [ handleEventHook def ],
        logHook            = myLogHook xmproc,
        --logHook            = dynamicLogWithPP $ xmobarPP { 
        startupHook        = myStartupHook
    }

-- A structure containing your configuration settings, overriding
-- fields in the default config. Any you don't override, will 
-- use the defaults defined in xmonad/XMonad/Config.hs
-- 
-- No need to modify this.

